// Hydrogen Sulphide gas detector using MQ-136 sensor. 
// Detects toxic gas given off from defective / overcharged Sealed Lead Acid batteries whilst charging
// Use good quality power supply that gives 5V output +/- 0.1% @ 500mA min.
// SSD needs to be common anode driven via ULN2803A. 
// This version gets rid of tripped LED as it's not really needed. Indication is given on the LCD instead and rapid flashing of SSD D.P.
// Use Piezo buzzer via 100 ohm resistor or magnetic buzzer via transistor on pin 11.
// Found some info the sensor outputs 100mV per 10ppm of H2S. 100mV = 20.48 ADC value (this will have to be rounded down to 20)
// This will depend on load resistor and also from sensor to sensor. Adjust thresholds accordingly as you will want the LED display
// to start indicating if gas is present and then trip when it continues to rise.

/*
a = 2
b = 3 (use 10 if using tone function)
c = 4
d = 6
e = 7
f = 9
g = 8
DP = 5

Pin 3 not used as this pin will be affected by the tone library. If not using a buzzer use 3 for segment 'b' on the SSD and have 10 as spare pin.

*/

#include <LCD_ST7032.h> // driver for the Midas displays i2C module.
#include <EEPROM.h>
#include "EEPROMAnything.h"
#include <avr/pgmspace.h> // library for keeping read only text strings in flash memory instead of copying to SRAM at startup.
#include <Wire.h> // library for i2C communications. Disables analog inputs A4 & A5 and allocates them as i2C pins SDA & SCL instead.
// address is 0x3C - change in LCD_ST7032.h header if libraries are updated.

LCD_ST7032 lcd;
char * const daysOfTheWeek[] PROGMEM = {"SUN", "MON", "TUE", "WED", "THU", "FRI", "SAT"};
char * const monthsOfyear[] PROGMEM =  {"JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};
int num;
int sensorPin = A0;
int sensorValue = 0;
int gasledPin = 13;
int heatingledPin = 5; // use decimal point on LED for this
int relayPin = 12; 
int buzzer = 11;
String installDate = "28-FEB-2018"; // sensor install date. Replace sensor every 5 years.

const int numReadings = 10;
int readings[numReadings];      // the readings from the analog input
int readIndex = 0;              // the index of the current reading
int total = 0;                  // the running total
int average = 0;   

struct config_t
{
    long alarm;
    int mode;
} configuration;

float sensor_voltage = 0.0; // sensor output voltage

unsigned long previousMillis = 0; // last time update
long interval = 10000; // send to serial every 10 seconds
unsigned long startTime;
unsigned long elapsedTime;
int tripped = 0;

  void sendtoSerial()

  {
   // diagnostics
  Serial.print(F("Sensor Value = "));
  Serial.print(sensorValue);
  Serial.println();
 
  sensor_voltage = (sensorValue *5.0) / 1023.0;
  Serial.print(F("Sensor output voltage = "));
  Serial.print(sensor_voltage);
  Serial.print("V");
  Serial.println();
 
  // end diagnostics
  }
  
  void lcdinit() // init specific to lcd used (midas displays MCCOG21605D6W) 20x2 characters

   {
   Wire.beginTransmission(0x3c);
   Wire.write(0x3c);
   delay(5);
   Wire.write(0x00);
   delay(5);
   Wire.write(0x38);
   delay(5);
   Wire.write(0x39);
   delay(5);
   Wire.write(0x14);
   delay(5);
   Wire.write(0x74);
   delay(5);
   Wire.write(0x54);
   delay(5);
   Wire.write(0x6f);
   delay(5);
   Wire.write(0x0c);
   delay(5);
   Wire.write(0x01);
   Wire.endTransmission();
    
   }
  
      void setup()
      {
          pinMode(heatingledPin, OUTPUT);
          pinMode(gasledPin, OUTPUT);
          pinMode(relayPin, OUTPUT);
          pinMode(buzzer, OUTPUT);
          pinMode(2, OUTPUT);
          pinMode(10, OUTPUT);
          pinMode(4, OUTPUT);
          pinMode(6, OUTPUT);
          pinMode(7, OUTPUT);
          pinMode(8, OUTPUT);
          pinMode(9, OUTPUT);
          
          //EEPROM_writeAnything(0, installDate); // comment out after running for first time
          Serial.begin(9600);      // sets the serial port to 9600
          digitalWrite(heatingledPin, HIGH); // indicate warming up
          lcd.begin();
          //lcd.setcontrast(24); //contrast value range is 0-63 (not needed now as display uses the init function instead)
          lcdinit();   // send additional initilization codes to LCD display
                 
          lcd.setCursor(0,0);
          lcd.print(F(" Gas detector V1.4"));
          Serial.print(F("Gas detector V1.4"));
          Serial.println();
          EEPROM_readAnything(0, installDate);
          Serial.print(F("Sensor Install Date "));
          Serial.print(installDate);
          Serial.println();          
          lcd.setCursor(1,0);
          lcd.print(F("2018 XXX XXX"));
          delay(5000);
          lcd.clear();
          lcd.setCursor(0,0);
          lcd.print(F("Install Date"));
          lcd.setCursor(1,0);
          lcd.print(installDate);
          delay(5000);
          lcd.clear();
          Serial.print(F("Sensor warming up, wait 2 mins"));
          Serial.println();
          lcd.setCursor(0,0);
          lcd.print(F("Sensor warming up..."));
          lcd.setCursor(1,0);
          lcd.print(F("Wait 2 mins"));
          delay(120000); // wait 2 mins
          
          // set all LED outputs and relay output off (relay will be a NC type)
          
          digitalWrite(heatingledPin, LOW);
          digitalWrite(gasledPin, LOW);
          digitalWrite(relayPin, LOW);
          digitalWrite(buzzer, LOW); // buzzer can be damaged by leaving 5V connected all the time so set low until buzzer is needed.
          lcd.clear();  
          tripped = 0; 
                 
            for (int thisReading = 0; thisReading < numReadings; thisReading++) 
            {
            readings[thisReading] = 0;
            }
       }


      void loop()
      {

        // subtract the last reading:
        total = total - readings[readIndex];
        // read from the sensor:
        readings[readIndex] = analogRead(sensorPin);
        // add the reading to the total:
         total = total + readings[readIndex];
        // advance to the next position in the array:
        readIndex = readIndex + 1;

        // if we're at the end of the array...
        if (readIndex >= numReadings) 
          {
        // ...wrap around to the beginning:
        readIndex = 0;
          }

        delay(1);
        sensorValue = total / numReadings;
        //sensorValue = analogRead(sensorPin);       // read analog input pin 0
        
              if ((sensorValue < 20) && (tripped == 0))    // if there is little or no gas detected blink decimal point (heating LED)
              {
                
                  digitalWrite(2, LOW);
                  digitalWrite(10, LOW);
                  digitalWrite(4, LOW);
                  digitalWrite(6, LOW);
                  digitalWrite(7, LOW);
                  digitalWrite(8, HIGH); // indicate dash along with blinking DP
                  digitalWrite(9, LOW);
                  digitalWrite(heatingledPin, LOW);
                  delay(1000);
                  digitalWrite(heatingledPin, HIGH);
                  delay(100);
                  lcd.setCursor(0,0);
                  lcd.print("H2S level Normal");
                  lcd.setCursor(1,0);
                  lcd.print("Chargers On");
                  lcd.setCursor(1,14);
                  lcd.print(sensorValue);
                  digitalWrite (relayPin, LOW); // 
                                    
              }
              else if ((sensorValue < 20) && (tripped == 1))

              {
                  digitalWrite(2, LOW);
                  digitalWrite(10, LOW);
                  digitalWrite(4, LOW);
                  digitalWrite(6, LOW);
                  digitalWrite(7, LOW);
                  digitalWrite(8, HIGH); // indicate dash along with blinking DP
                  digitalWrite(9, LOW);
                
                  digitalWrite(heatingledPin, HIGH);
                  delay(200);
                  digitalWrite(heatingledPin, LOW);
                  delay(200);
                  lcd.setCursor(0,0);
                  lcd.print("Tripped ");
                  elapsedTime = millis() - startTime;
                  lcd.print(elapsedTime /1000 /60); // divide milliseconds to get seconds then divide to get mins
                  lcd.print(" mins ago");
                  lcd.setCursor(1,0);
                  lcd.print("Chargers Off");
                  lcd.setCursor(1,14);
                  lcd.print(sensorValue);
                  digitalWrite (relayPin, HIGH); // turn off charger(s) - use a normally closed relay
                                    
              }
              
              else
              {
                  if (sensorValue >= 160)               // the value of the gas measurement is displayed in rough scale of ppm /10 
                  {num = 9;}
                  else if (sensorValue >= 120)
                  {num = 8;}
                  else if (sensorValue >= 95)
                  {num = 7;}
                  else if (sensorValue >= 70)
                  {num = 6;} // trip alarm when 6 or higher is displayed by LED.
                  else if (sensorValue >= 55)
                  {num = 5;
                  lcd.setCursor(1,14);
                  lcd.print(sensorValue);}
                  else if (sensorValue >= 47)
                  {num = 4;
                  lcd.setCursor(1,14);
                  lcd.print(sensorValue);}
                  else if (sensorValue >= 40)
                  {num = 3;
                  lcd.setCursor(1,14);
                  lcd.print(sensorValue);} 
                  else if (sensorValue >= 30)
                  {num = 2;
                  lcd.setCursor(1,14);
                  lcd.print(sensorValue);}
                  else if (sensorValue >= 21)
                  {num = 1;
                  lcd.setCursor(1,14);
                  lcd.print(sensorValue);}   
              }
              
              if (sensorValue > 70)               
              {
                  digitalWrite (gasledPin, HIGH); // indicate dangerous levels of toxic gas
                  digitalWrite (relayPin, HIGH); // turn off charger(s) - use a normally closed relay
                  tone(buzzer, 3850); // make annoying sound at 3.8khz to get attention  
                  lcd.setCursor(0,0);
                  lcd.print("H2S Gas Warning     ");
                  lcd.setCursor(1,0);
                  lcd.print("Chargers Off");
                  lcd.setCursor(1,14);
                  lcd.print(sensorValue);
                  tripped = 1;
                  startTime = millis();
                  
              }  
                else
                 {
                  digitalWrite (gasledPin, LOW); // if gas returns to normal turn off gas LED but leave chargers off 
                 }
        
                  switch (num)
                  {
                  case 9:
                  digitalWrite(2, HIGH);
                  digitalWrite(10, HIGH);
                  digitalWrite(4, HIGH);
                  digitalWrite(6, HIGH);
                  digitalWrite(7, LOW);
                  digitalWrite(8, HIGH);
                  digitalWrite(9, HIGH);
                  break;
                  case 8:
                  digitalWrite(2, HIGH);
                  digitalWrite(10, HIGH);
                  digitalWrite(4, HIGH);
                  digitalWrite(6, HIGH);
                  digitalWrite(7, HIGH);
                  digitalWrite(8, HIGH);
                  digitalWrite(9, HIGH);
                  break;
                  case 7:
                  digitalWrite(2, HIGH);
                  digitalWrite(10, HIGH);
                  digitalWrite(4, HIGH);
                  digitalWrite(6, LOW);
                  digitalWrite(7, LOW);
                  digitalWrite(8, LOW);
                  digitalWrite(9, HIGH);
                  break;
                  case 6:
                  digitalWrite(2, HIGH);
                  digitalWrite(10, LOW);
                  digitalWrite(4, HIGH);
                  digitalWrite(6, HIGH);
                  digitalWrite(7, HIGH);
                  digitalWrite(8, HIGH);
                  digitalWrite(9, HIGH);
                  break;
                  case 5:
                  digitalWrite(2, HIGH);
                  digitalWrite(10, LOW);
                  digitalWrite(4, HIGH);
                  digitalWrite(6, HIGH);
                  digitalWrite(7, LOW);
                  digitalWrite(8, HIGH);
                  digitalWrite(9, HIGH);
                  break;
                  case 4:
                  digitalWrite(2, LOW);
                  digitalWrite(10, HIGH);
                  digitalWrite(4, HIGH);
                  digitalWrite(6, LOW);
                  digitalWrite(7, LOW);
                  digitalWrite(8, HIGH);
                  digitalWrite(9, HIGH);
                  break;
                  case 3:
                  digitalWrite(2, HIGH);
                  digitalWrite(10, HIGH);
                  digitalWrite(4, HIGH);
                  digitalWrite(6, HIGH);
                  digitalWrite(7, LOW);
                  digitalWrite(8, HIGH);
                  digitalWrite(9, LOW);
                  break;
                  case 2:
                  digitalWrite(2, HIGH);
                  digitalWrite(10, HIGH);
                  digitalWrite(4, LOW);
                  digitalWrite(6, HIGH);
                  digitalWrite(7, HIGH);
                  digitalWrite(8, HIGH);
                  digitalWrite(9, LOW);
                  break;
                  case 1:
                  digitalWrite(2, LOW);
                  digitalWrite(10, HIGH);
                  digitalWrite(4, HIGH);
                  digitalWrite(6, LOW);
                  digitalWrite(7, LOW);
                  digitalWrite(8, LOW);
                  digitalWrite(9, LOW);
                  break;
                  }
                  
                                                  
                  unsigned long currentMillis = millis();

                  if(currentMillis - previousMillis > interval) 
                   {
                    previousMillis = currentMillis;  
                    sendtoSerial();
                   }
 
        
      }
